/**
 * 
 */

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.Spring;
import javax.swing.SpringLayout;
import javax.swing.UIManager;
import javax.swing.BorderFactory;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import java.net.URL;
import java.util.Observable;
import java.util.Observer;
import java.io.File;
import java.io.IOException;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class ICSManagementView extends JPanel implements Observer {
    private JPanel rightPane ;
    private static boolean DEBUG = false;
    JSplitPane sp;
    ICSModel im;
    RightValuePanel rvp;

    //Optionally play with line styles.  Possible values are
    //"Angled" (the default), "Horizontal", and "None".
    private static boolean playWithLineStyle = false;
    private static String lineStyle = "Horizontal";
    
    //Optionally set the look and feel.
    private static boolean useSystemLookAndFeel = false;
	

    public ICSManagementView() throws SAXException, IOException, ParserConfigurationException, TransformerConfigurationException, TransformerException  {
        super(new GridLayout(1,0));
        
        setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
        ICSMDSPanel imp = new ICSMDSPanel();
    	add(imp);
     	
     	ICSTreePanel itp = new ICSTreePanel();
     	rvp = new RightValuePanel();
     	     	
        rightPane = new JPanel(new CardLayout());
    	rightPane.add(new JPanel(), "Blank");
    	rightPane.add(rvp, "ElementValue");
         
        //	 SplitPane 
        JPanel p0 = new JPanel();
        p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
        JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, itp, rightPane);
        sp.setDividerLocation(200);
       
        p0.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

        p0.add(sp);
        add(p0);
        ICSGenerationPanel igp = new ICSGenerationPanel();
        add(igp);
        ICSDisplayPanel idp = new ICSDisplayPanel();
    	add(idp);
//      Create the Model
        im = new ICSModel();
        
        
        im.addObserver(itp);
        im.addObserver(this);
        im.addObserver(rvp);
        im.addObserver(igp);
        im.addObserver(idp);
        
//      Set the Model
        imp.setICSModel(im);
        itp.setICSModel(im);
        igp.setICSModel(im);
        rvp.setICSModel(im);
       
    }
   
    

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof ICSModel) {
		
	    if(arg instanceof MVCArg) {
		if("node".equals(((MVCArg)arg).getName())) {
		    Object obj = ((MVCArg)arg).getOValue();
			CardLayout cl = (CardLayout)(rightPane.getLayout());
			cl.show(rightPane, "ElementValue");
		}
		else if("nodeBlank".equals(((MVCArg)arg).getName())) {
		    CardLayout cl = (CardLayout)(rightPane.getLayout());
		    cl.show(rightPane, "Blank");
		}
		else if("init".equals(((MVCArg)arg).getName())) {
		    CardLayout cl = (CardLayout)(rightPane.getLayout());
		    cl.show(rightPane, "Blank");
		}
	    }
	}
    }

    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     * @throws ParserConfigurationException
     * @throws IOException
     * @throws SAXException
     */
    private static void createAndShowGUI() throws SAXException, IOException, ParserConfigurationException, TransformerException, TransformerConfigurationException {
        if (useSystemLookAndFeel) {
            try {
                UIManager.setLookAndFeel(
                    UIManager.getSystemLookAndFeelClassName());
            } catch (Exception e) {
                System.err.println("Couldn't use system look and feel.");
            }
        }
          
        //Make sure we have nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(true);

        //Create and set up the window.
        JFrame frame = new JFrame("ICSGENERATOR");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //Create and set up the content pane.
        ICSManagementView newContentPane = new ICSManagementView();    
        newContentPane.setOpaque(true); //content panes must be opaque
        frame.setContentPane(newContentPane);

        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }

    public static void main(String[] args) throws SAXException, IOException, ParserConfigurationException, TransformerConfigurationException, TransformerException{
        //Schedule a job for the event-dispatching thread:
        //creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                try {
					createAndShowGUI();
				} catch (SAXException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (ParserConfigurationException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (TransformerException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();

				}
            }
        });
    }
}

